<?xml version="1.0" encoding="UTF-8"?>
<stylesheet xmlns="http://www.w3.org/1999/XSL/Transform"
	xmlns:seaf="http://seaf.sf.net/seaf"
	xmlns:per="http://seaf.sf.net/seaf-persistence"
	version="2.0">
<import href="functions/SeafPersistenceBusinessFunctions.xsl" />
<output method="xml" indent="yes"
	doctype-system="http://hibernate.sourceforge.net/hibernate-mapping-3.0.dtd"
	doctype-public="-//Hibernate/Hibernate Mapping DTD//EN" />

<param name="merged-file-name" />
<variable name="merged-persistence-file" select="document($merged-file-name)" />

<variable name="entities" select="$merged-persistence-file//per:persistence//per:entity" />
<variable name="relations" select="$merged-persistence-file//per:persistence//per:relation" />

<template match="/">
	<apply-templates select="//per:persistence//per:entity" />
</template>

<template match="//per:persistence//per:entity">
	<variable name="entity-name" select="@name" />
	<variable name="entity-class" select="seaf:type-class(@name)" />
	<variable name="entity-dir" select="seaf:type-dir(per:hbm-xml-full-type(.))" />
	<variable name="entity-package" select="seaf:type-package(per:entity-full-type(.))" />
	<variable name="fields" select="per:entity-fields(.,())" />

	<result-document href="{$entity-dir}/{$entity-class}.hbm.xml">
		<element name="hibernate-mapping" namespace="">
			<attribute name="package">
				<value-of select="$entity-package" />
			</attribute>
			<if test="$fields/per:binary">
				<element name="typedef" namespace="">
					<attribute name="name">blob_type</attribute>
					<attribute name="class">org.springframework.orm.hibernate3.support.BlobByteArrayType</attribute>
				</element>
			</if>
			<element name="class" namespace="">
				<attribute name="name">
					<value-of select="$entity-class" />
				</attribute>
				<attribute name="table">
					<value-of select="per:table-name($entity-class)" />
				</attribute>
				<!-- primary key -->
				<call-template name="pk-template" />
				<!-- version for optimistic locking -->
				<call-template name="version-template">
					<with-param name="fields" select="$fields" />
				</call-template>
				<!-- own fields -->
				<apply-templates select="$fields[not(@status='primary-key-assigned' 
					or @status='primary-key-generated' or @status='transient'
					or @status='criteria-only' or @status='version')]">
					<with-param name="entity-name" select="$entity-name" />
				</apply-templates>
				<!-- relations to masters -->
				<apply-templates select="$relations/per:detail[@entity-name=$entity-name]" />
				<!-- relations to details -->
				<apply-templates select="$relations/per:master[@entity-name=$entity-name]" />
			</element>
		</element>
	</result-document>
</template>

<template name="pk-template">
	<variable name="entity-name" select="@name" />
	<variable name="pk-field" select="per:persitent-object-pk-field(.)" />
	<element name="id" namespace="">
		<attribute name="name">
			<value-of select="$pk-field/@name" />
		</attribute>
		<attribute name="column">
			<value-of select="per:column-name($pk-field/@name)" />
		</attribute>
		<choose>
		<when test="$pk-field/@status='primary-key-assigned'">
			<!-- Custom PK column -->
			<if test="$pk-field/per:string">
				<attribute name="length">
					<value-of select="$pk-field/per:string/@length" />
				</attribute>
			</if>
		</when>
		<otherwise>
			<!-- Default PK column -->
			<attribute name="type">long</attribute>
			<element name="generator" namespace="">
				<attribute name="class">org.hibernate.id.enhanced.SequenceStyleGenerator</attribute>
				<element name="param" namespace="">
					<attribute name="name">sequence_name</attribute>
					<value-of select="per:sequence-name($entity-name)" />
				</element>
				<element name="param" namespace="">
					<attribute name="name">initial_value</attribute>
					<text>100</text>
				</element>
			</element>
		</otherwise>
		</choose>
	</element>
</template>

<template name="version-template">
<param name="fields" />
	<variable name="version-field" select="$fields[@status='version']" />
	<element name="version" namespace="">
		<apply-templates select="$version-field/@name" mode="copy" />
		<attribute name="column">
			<value-of select="per:column-name($version-field/@name)" />
		</attribute>
		<attribute name="type">long</attribute>
	</element>
</template>

<template match="per:field">
<param name="entity-name" />
<choose>
<when test="per:enum | per:code-table">
	<element name="many-to-one" namespace="">
		<apply-templates select="@name" mode="copy" />
		<attribute name="column">
			<value-of select="per:column-name(@name)" />
		</attribute>
		<attribute name="foreign-key">
			<value-of select="per:fk-name($entity-name,@name)" />
		</attribute>
		<call-template name="status-template" />
	</element>
</when>
<otherwise>
	<element name="property" namespace="">
		<apply-templates select="@name" mode="copy" />
		<attribute name="column">
			<value-of select="per:column-name(@name)" />
		</attribute>
		<choose>
		<when test="per:string">
			<attribute name="length">
				<value-of select="per:string/@length" />
			</attribute>
		</when>
		<when test="per:text">
			<attribute name="type">text</attribute>
		</when>
		<when test="per:date">
			<attribute name="type">calendar_date</attribute>
		</when>
		<when test="per:binary">
			<attribute name="type">blob_type</attribute>
		</when>
		</choose>
		<call-template name="status-template" />
	</element>
</otherwise>
</choose>
</template>

<template name="status-template">
	<if test="@status='mandatory' or @status='unique'">
		<attribute name="not-null">true</attribute>
	</if>
	<if test="@status='unique'">
		<attribute name="unique">true</attribute>
	</if>
</template>

<template match="per:detail">
	<variable name="entity-name" select="@entity-name" />
	<variable name="property-name" select="per:relation-property-name(.,..)" />
	<variable name="column-name-base" select="concat($property-name,'Id')" />
	<element name="many-to-one" namespace="">
		<attribute name="name">
			<value-of select="$property-name" />
		</attribute>
		<attribute name="column">
			<value-of select="per:column-name($column-name-base)" />
		</attribute>
		<attribute name="foreign-key">
			<value-of select="per:fk-name($entity-name,$property-name)" />
		</attribute>
		<if test="not(../per:master/@status='optional')">
			<attribute name="not-null">true</attribute>
		</if>
		<if test="../@type='one-to-one'">
			<attribute name="unique">true</attribute>
		</if>
	</element>
</template>

<template match="per:master[../@type='one-to-many']">
	<variable name="property-name" select="per:relation-property-name(.,..)" />
	<variable name="opposite-property-name" 
		select="per:opposite-relation-property-name(.,..)" />
	<variable name="column-name-base" select="concat($opposite-property-name,'Id')" />
	<variable name="related-entity-name" select="per:related-entity-name(.,..)" />
	<variable name="related-type" 
		select="per:entity-full-type($entities[@name=$related-entity-name])" />

	<element name="bag" namespace="">
		<attribute name="name">
			<value-of select="$property-name" />
		</attribute>
		<attribute name="inverse">true</attribute>
		<attribute name="cascade">all, delete-orphan</attribute>
		<element name="key" namespace="">
			<attribute name="column">
				<value-of select="per:column-name($column-name-base)" />
			</attribute>
		</element>
		<element name="one-to-many" namespace="">
			<attribute name="class">
				<value-of select="$related-type" />
			</attribute>
		</element>
	</element>
</template>

<template match="per:master[../@type='one-to-one']">
	<variable name="property-name" select="per:relation-property-name(.,..)" />
	<variable name="related-property-name"
		select="per:opposite-relation-property-name(.,..)" />

	<element name="one-to-one" namespace="">
		<attribute name="name">
			<value-of select="$property-name" />
		</attribute>
		<attribute name="cascade">all</attribute>
		<attribute name="property-ref">
			<value-of select="$related-property-name" />
		</attribute>
	</element>
</template>

</stylesheet>
